#include <Adafruit_NeoPixel.h>
#include <EEPROM.h>
#ifdef __AVR__
#include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

#define LED_PIN    6
#define LED_COUNT 30
#define DAYS_OVER_AVG 10;
#define ADD_PIN 3
#define CLR_PIN 4
#define AVG_PIN 5
#define RAINBOW 2
#define MAX_TASKS 30

void addTask();
void clearTasks();
void visualize();
void theaterChase(int wait);


Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);
int addr = 0;
char nTasks = 0;
char auxTasks = 0;
bool prevAdd = 0;
bool prevClr = 0;
bool prevAvg = 0;
bool prevBow = 0;
bool avgOn = false;
bool rainbowOn = false;


void setup() {

  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(50); // Set BRIGHTNESS to about 1/5 (max = 255)
  pinMode(ADD_PIN, INPUT);
  pinMode(CLR_PIN, INPUT);
  pinMode(AVG_PIN, INPUT);
  randomSeed(analogRead(0));
  Serial.begin(9600);
}


// loop() function -- runs repeatedly as long as board is on ---------------

void loop() {
  //  // Fill along the length of the strip in various colors...
  //  colorWipe(strip.Color(255,   0,   0), 50); // Red
  int currAdd = digitalRead(ADD_PIN);
  int currClr = digitalRead(CLR_PIN);
  int currAvg = digitalRead(AVG_PIN);
  int currBow = digitalRead(RAINBOW);
  if (currAdd && !prevAdd) {
    addTask();
  } else if (currClr && !prevClr) {
    clearTasks();
  } else if (currAvg && !prevAvg) {
    avgOn = !avgOn;
    if (!avgOn) {
      strip.clear();
    }
  } else if (currBow && !prevBow) {
    rainbowOn = !rainbowOn;
    if (!rainbowOn) {
      strip.clear();
    }
  }
  visualize();
  prevAdd = currAdd;
  prevClr = currClr;
  prevAvg = currAvg;
  prevBow = currBow;

}

void addTask() {
  /*
     TODO: drop down the pixel from the top in a cool way
  */

  if (nTasks == MAX_TASKS) {
    clearTasks();
  } else {
    nTasks++;
  }
  for (int i = MAX_TASKS - 1; i > nTasks; i--) {
    strip.setPixelColor(i, strip.Color((int)random(150), (int)random(150), (int)random(150)));
    strip.show();
    delay(50);
    strip.setPixelColor(i, strip.Color(0, 0, 0));
  }


}

void clearTasks() {
  /*TODO: add value to eeprom in first 10 slots
          change the boolean over10 flag to true if addr = 9
          set all LEDs to OFF in a cool way
  **/
  if (addr == DAYS_OVER_AVG - 1) {
    EEPROM.write(addr + 1, true);
  }
  addr = (addr + 1) % DAYS_OVER_AVG;
  EEPROM.write(addr, nTasks);
  for (int i = nTasks - 1; i > 0; i--) {
    strip.setPixelColor(0, strip.Color(0, 0, 0));
    strip.show();
    delay(50);
    for (int j = 0; j < i; j++) {
      strip.setPixelColor(j, strip.getPixelColor(j + 1));
      strip.setPixelColor(j + 1, strip.Color(0, 0, 0));
      strip.show();
      delay(10);
    }
    delay(50);
  }
  strip.setPixelColor(0, strip.Color(0, 0, 0));
  strip.show();

  nTasks = 0;
}


void visualize() {
  /*
     TODO: visualize either the number of tasks or the number of tasks compared to the average
  */
  if (rainbowOn) {
    bool keepGoing = true;
    int hue = 0;
    prevBow = true;
    while (keepGoing) {
      int currBow = digitalRead(RAINBOW);
      for (int i = 0; i < nTasks; i++) {
        strip.setPixelColor(i, strip.ColorHSV((hue + 1000*i) % 65536));
      }
      strip.show();
      delay(10);
      hue = (hue + 100) % 65536;

      keepGoing = !(currBow && !prevBow);
      prevBow = currBow;
    }
    rainbowOn  = false;
    

  } else if (avgOn) {
    /*
       calculate average over nDays. round it to nearest int. If the current total is more than the average, display average in green, show the rest in blue.
       If the current total is less than the average, show the current total in green and the difference in red
    */
    int sum = 0;
    int roundAvg = 0;
    int a = DAYS_OVER_AVG;
    if (EEPROM.read(a)) {
      for (int i = 0; i < a; i++) {
        sum = sum + EEPROM.read(i);
      }
      float avg = (1.0 * sum) / (a);
      roundAvg = (int)avg;

    } else {
      for (int i = 0; i <= addr; i++) {
        sum = sum + EEPROM.read(i);
      }
      float avg = (1.0 * sum) / (addr);
      roundAvg = (int)avg;
    }
    Serial.println(sum);
    if (nTasks < roundAvg) {
      for (int i = 0; i < nTasks; i++) {
        strip.setPixelColor(i, strip.Color(0, 150, 0));
      }
      for (int i = nTasks; i < roundAvg; i++) {
        strip.setPixelColor(i, strip.Color(150, 0, 0));
      }
      strip.show();
    } else {
      for (int i = 0; i < roundAvg; i++) {
        strip.setPixelColor(i, strip.Color(0, 150, 0));
      }
      for (int i = roundAvg; i < nTasks; i++) {
        strip.setPixelColor(i, strip.Color(0, 0, 150));
      }
      strip.show();
    }

  } else {

    for (int i = 0; i < nTasks; i++) {
      strip.setPixelColor(i, strip.Color(0, 150, 0));
    }
    strip.show();
  }
}
